#!/usr/bin/env php
<?php
#############################################################################
# PROGRAM
#     progstat-0.2.php
#
# SYNOPSIS
#     Used to view current download statistics for posted programs
#     on various sites.
#
# USAGE
#     progstat-0.2.php
#
# OPTIONS
#     None
#
# MODULES/FILES REQUIRED
#     None
#
# PROGRAMS REQUIRED
#     None
#
# AUTHOR
#     Steve Dickinson
#
#     email: dsdickinson@mac.com
#      site: www.codeuniversity.com
#
# DEVELOPMENT DATES
#     01.15.10
#
# RELEASE DATE
#     01.25.10
#
# COMMENTS
#     None
#
# REVISIONS
#     None
#############################################################################
# LICENSE
#############################################################################
# progstat - Program Statistics
# Copyright (c) 2010 Steve Dickinson. 
# All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License (see COPYING)
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
# VARIABLES
#############################################################################
$version = "0.2";

date_default_timezone_set('America/New_York');
$date = date(YmdHis);

# Add the direct page url's where the software can be found.
# Currently softpedia and sourceforge are the two supported hosts.
$host_urls = array (
		"softpedia" => array (
			"http://mac.softpedia.com/get/Dashboard-Widgets/Network-Security/FreeDiskO.shtml",
			"http://mac.softpedia.com/get/Dashboard-Widgets/Network-Security/ChmodPro.shtml",
			"http://mac.softpedia.com/get/Dashboard-Widgets/Network-Security/SysfO.shtml"
			),
		"sourceforge" => array (
			"http://sourceforge.net/projects/freedisko/files/",
			"http://sourceforge.net/projects/sysfo/files/",
			"http://sourceforge.net/projects/chmodpro/files/",
			"https://sourceforge.net/projects/progstat/files/"
			)
		);
	
# Log file where the count records are stored and appended to for historical review.
$HOME	= getenv("HOME");
if (! empty($HOME)) {
    $myFile = $HOME . "/progstat.txt";
} else {
    $myFile = "/tmp/progstat.txt";
}

#############################################################################
# MAIN
#############################################################################
$fh = fopen($myFile, 'a') or die("Can't open file $myFile.");
echo "#=====================================================================================================\n";
echo "= progstat v$version                                                                                      =\n";
echo "= Copyright (c) 2010 Steve Dickinson                                                                 =\n";
echo "= https://sourceforge.net/projects/progstat/                                                         =\n";
#echo "#=====================================================================================================\n\n";
foreach ($host_urls as $host=>$urls) {
    if (!preg_match("/^softpedia$/", $host) &&
        !preg_match("/^sourceforge$/", $host)) {continue;}
    echo "#=====================================================================================================\n";
    echo "# $host\n";
    echo "#=====================================================================================================\n";
    foreach ($urls as $url ) {
	$scrape = scrape($url);
	$title = get_title($host, $scrape) . "\n";
	echo $title;
	do_download_counts($host, $scrape);
	echo "\n";
    }
}
fclose($fh);

exit(0);

#############################################################################
# FUNCTIONS
#############################################################################
#============================================================================
# Scrape screen
#============================================================================
function scrape($url) {
    $raw = file_get_contents($url);
    $newlines = array("\t","\n","\r","\x20\x20","\0","\x0B");

    $content = str_replace($newlines, "", html_entity_decode($raw));

    return ($content);
}

#============================================================================
# Get title from page's title sections
#============================================================================
function get_title($host, $scrape) {
    switch ($host) {
        case "softpedia":
	    $pattern = "/<title>(.*)<\/title>/";
	    $pattern_remove = "/^Download /";
            break;
        case "sourceforge":
	    $pattern = "/<title>(.*)<\/title>/";
	    $pattern_remove = "/^Browse /";
            break;
        default:
            break;
    }
    
    preg_match_all("$pattern", $scrape,  $title_text);
    $title_org = $title_text[1][0];
    $title = preg_replace($pattern_remove, "", $title_org);
    if ($host == "softpedia") {
        $title = preg_replace("/ - Softpedia/", "", $title);
    }

    return ($title);
}

#============================================================================
# Get download counts
#============================================================================
function do_download_counts($host, $scrape) {
    global $date, $fh, $title;
    $all_files = 0;

    switch ($host) {
        case "softpedia":
	    $pattern = "/Downloads:.*?\>(\d+)</";
    	    preg_match_all("$pattern", $scrape,  $downloads);
    	    $download_count = $downloads[1][0];
	    list($name,$desc,$place) = split(" - ", $title);
	    echo "   ";
	    printf ("%' -62.62s%' 37.37s\n", $name, $download_count);
	    fwrite($fh, "$date,$host,$name,$download_count\n");
            break;
        case "sourceforge_org":
            $start = strpos($scrape,'<table class="filesTable treeTable"');
            $end = strpos($scrape,'</table>',$start) + 8;
            $table = substr($scrape,$start,$end-$start);

            preg_match_all("|<tr(.*)</tr>|U",$table,$rows);
            foreach ($rows[0] as $row){
                if ((strpos($row,'<th')===false)){
                    preg_match_all("|<td(.*)</td>|U",$row,$cells);
                    if (preg_match("/All Files/i", $row)) {
			$all_files = 1;
			continue;
		    }	
		    if ($all_files == 1) {
                        $name = strip_tags($cells[0][0]);
                        $download_count = strip_tags($cells[0][4]);
	 	        echo "   ";
			printf ("%' -62.62s%' 37.37s\n", $name, $download_count);
			fwrite($fh, "$date,$host,$name,$download_count\n");
		    }
                }
            }
            break;
        case "sourceforge":
            $start = strpos($scrape,'<table class="files_list"');
            $end = strpos($scrape,'</table>',$start) + 8;
            $table = substr($scrape,$start,$end-$start);

            preg_match_all("|<tbody>(.*)</tbody>|U",$table,$sub_table);
            $sub_table_match = $sub_table[0][0];
            preg_match_all("|<tr(.*)</tr>|U",$sub_table_match,$rows);
            foreach ($rows[0] as $row){
	        preg_match("/tr title=\"(.*?)\"/", $row, $matches);
		$name = $matches[1];
	        preg_match("/title=\"([0-9]+) downloads/", $row, $matches);
		$download_count = $matches[1];
	 	echo "   ";
		printf ("%' -62.62s%' 37.37s\n", $name, $download_count);
		fwrite($fh, "$date,$host,$name,$download_count\n");
            }
            break;
        default:
            break;
    }
}
?>
